import { Handle, HandleType, Position } from 'reactflow';
import { QueryGraphEdgeHandle, toHandleId } from '../../..';
import { tailwindColors } from 'config';

const rightArrow = '0,0 8,5 0,10';
const leftArrow = '8,0 0,5 8,10';
const square = '9,9 1,9 1,1 9,1';
const getArrow = {
  right: rightArrow,
  left: leftArrow,
  both: square,
};
export type RelationshipHandleArrowType = 'right' | 'left' | 'both';

type Props = { id: QueryGraphEdgeHandle; type: HandleType; point: RelationshipHandleArrowType; onDoubleClick?: () => void };

export const LeftHandle = (props: Props) => {
  const offset = 15;

  return (
    <Handle
      id={toHandleId(props.id)}
      type={props.type}
      position={Position.Left}
      className=""
      style={{ transform: `translate(${offset}px, -3px)` }}
      onDoubleClickCapture={(e) => {
        e.preventDefault();
        e.stopPropagation();
        if (props.onDoubleClick) props.onDoubleClick();
      }}
    >
      <svg className="pointer-events-none" height={10} style={{ transform: `translate(-2px, -3px)` }}>
        <polygon points={getArrow[props.point]} fill={tailwindColors.relation[200]} />
      </svg>
    </Handle>
  );
};

export const RightHandle = (props: Props) => {
  const offset = -13;

  return (
    <Handle
      id={toHandleId(props.id)}
      type={props.type}
      position={Position.Right}
      className=""
      style={{ transform: `translate(${offset}px, -3px)` }}
      onDoubleClickCapture={(e) => {
        e.preventDefault();
        e.stopPropagation();
        if (props.onDoubleClick) props.onDoubleClick();
      }}
    >
      <svg height={10} className="pointer-events-none" style={{ transform: `translate(-2px, -3px)` }}>
        <polygon points={getArrow[props.point]} fill={tailwindColors.relation[200]} />
      </svg>
    </Handle>
  );
};
