/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { GeoNodeType, Command, GeoEdgeType, GeoGraphType, Coordinates } from '../types';
import { calcAverageCoordinates } from '../utils';
import * as d3 from 'd3';

export class AggregateCommand implements Command {
  /**
   * Groups nodes by a provided attribute
   * Computes average coordinates for new grouped node
   * Updates edge coordinates accordingly
   */
  private dimension: string;

  constructor(dimension: string) {
    this.dimension = dimension;
  }

  execute(data: GeoGraphType) {
    const graph: GeoGraphType = { ...data };
    graph.nodes = aggregateNodes(graph, this.dimension);
    graph.edges = aggregateEdges(graph);
    return graph;
  }
}

const aggregateNodes = (graph: GeoGraphType, dimension: string): GeoNodeType[] => {
  const nodesByAttribute: Map<string, GeoNodeType[]> = d3.group(graph.nodes, (node: GeoNodeType) => eval(`node.${dimension}`));

  const bundleByAttribute: GeoNodeType[] = [];

  nodesByAttribute.forEach((group) => {
    const lat: number[] = [];
    const long: number[] = [];
    const locations: string[] = [];

    group.forEach((location: GeoNodeType) => {
      const coordinates: Coordinates = location.attributes.coordinates;
      lat.push(coordinates[0]);
      long.push(coordinates[1]);
      locations.push(location.id);
    });

    const average: [number, number] = calcAverageCoordinates(lat, long);
    const name: string = eval(`group[0].${dimension}`);

    bundleByAttribute.push({
      id: name,
      attributes: {
        coordinates: average,
        locations: locations,
      },
    });
  });

  return bundleByAttribute;
};

const aggregateEdges = (data: GeoGraphType): GeoEdgeType[] => {
  const locations: { [key: string]: string[] } = data.nodes.reduce((locations: any, node) => {
    locations[node.id] = node.attributes.locations;
    return locations;
  }, {});

  const bundles: GeoEdgeType[] = [];

  data.edges.forEach((edge: GeoEdgeType) => {
    const from_location: string | undefined = Object.keys(locations).find((state) => locations[state].includes(edge.from));

    const to_location: string | undefined = Object.keys(locations).find((state) => locations[state].includes(edge.to));

    if (from_location && to_location) {
      if (from_location === to_location) return; // edge is within group

      const from_coordinates: Coordinates | undefined = data.nodes.find((node: GeoNodeType) => node.id === from_location)?.attributes
        .coordinates;

      const to_coordinates: Coordinates | undefined = data.nodes.find((node: GeoNodeType) => node.id === to_location)?.attributes
        .coordinates;

      const newEdge: GeoEdgeType = {
        ...edge,
        from: from_location,
        to: to_location,
        from_coordinates: from_coordinates,
        to_coordinates: to_coordinates,
      };

      bundles.push(newEdge);
    }
  });

  return bundles;
};
